﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;

namespace CustomDateTime
{
    public partial class PHDateTimeEditor : UserControl, IProvidesEmbeddableEditor, INotifyPropertyChanged
    {
        public PHDateTimeEditor()
        {
            InitializeComponent();

            this.Height = this.ultraMaskedEdit1.Height;
            ultraMaskedEdit1.NullText = String.Empty;
        }

        #region ISupportEditCommands

        public void Copy()
        {
            ultraMaskedEdit1.Copy();
        }

        public void Cut()
        {
            ultraMaskedEdit1.Cut();
        }

        public void Paste()
        {
            ultraMaskedEdit1.Paste();
        }

        public void SelectAll()
        {
            ultraMaskedEdit1.SelectAll();
        }

        #endregion

        #region IProvidesEmbeddableEditor

        public EmbeddableEditorBase Editor
        {
            get { return ultraMaskedEdit1.Editor; }
        }

        #endregion

        #region INotifyPropertyChanged

        public event System.ComponentModel.PropertyChangedEventHandler PropertyChanged;

        protected void NotifyPropertyChanged(string name)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new System.ComponentModel.PropertyChangedEventArgs(name));
            }
        }

        #endregion

        #region Properties

        public DateTime Date
        {
            get
            {
                return NullableDate.HasValue ? NullableDate.Value : DefaultNullDate;
            }
            set
            {
                if (value == DefaultNullDate)
                {
                    NullableDate = null;
                }
                else
                {
                    NullableDate = value;
                }
                NotifyPropertyChanged("Date");
                NotifyPropertyChanged("NullableDate");
            }
        }

        public DateTime? NullableDate
        {
            get
            {
                return (DateTime?)Value;
            }
            set
            {
                if (value == null && Value == null) return; // no changes
                if (value.HasValue && value.Value == Value as DateTime?) return; // no changes

                if (value.HasValue && value.Value == DefaultNullDate)
                {
                    Value = null;
                }
                else
                {
                    Value = value;
                }
                NotifyPropertyChanged("Date");
                NotifyPropertyChanged("NullableDate");
            }
        }

        private object _value;
        private object Value
        {
            get
            {
                return _value;
            }
            set
            {
                _value = value;
                UpdateText();
                UpdateMonthCalendar();
                NotifyPropertyChanged("Date");
                NotifyPropertyChanged("NullableDate");
            }
        }

        /// <summary>
        /// Date to use has a null value
        /// Default: 0001-01-01
        /// </summary>
        public DateTime DefaultNullDate { get; set; }

        public bool ReadOnly
        {
            get
            {
                return ultraMaskedEdit1.ReadOnly;
            }
            set
            {
                if (this.Disposing || this.IsDisposed)
                {
                    return;
                }

                ultraMaskedEdit1.ReadOnly = value;

                if (value)
                {
                    ultraMaskedEdit1.ButtonsRight[0].Visible = false;
                }
                else
                {
                    ultraMaskedEdit1.ButtonsRight[0].Visible = true;
                }
            }
        }

        #endregion

        #region Private Properties

        private bool _isValid = true;

        #endregion

        #region Private Methods

        private void UpdateValue()
        {
            if ((ultraMaskedEdit1.Value is System.DBNull))
            {
                Value = null;
                OnValueChanged();
                _isValid = true;
            }
            else if (((string)ultraMaskedEdit1.Value).IsNullOrWhiteSpace())
            {
                Value = null;
                OnValueChanged();
                _isValid = true;
            }
            else if (((string)ultraMaskedEdit1.Value).Length == 10)
            {
                DateTime date;
                if (DateTime.TryParseExact((string)ultraMaskedEdit1.Value, "yyyy-MM-dd", null, System.Globalization.DateTimeStyles.None, out date))
                {
                    if (date == DefaultNullDate)
                    {
                        _isValid = false;
                    }
                    else
                    {
                        Value = date.Date;
                        OnValueChanged();
                        _isValid = true;
                    }
                }
                else
                {
                    _isValid = false;
                }
            }
            else
            {
                _isValid = false;
            }

            this.Validate(false);
        }

        private void UpdateText()
        {
            if (NullableDate.HasValue)
            {
                ultraMaskedEdit1.Text = Date.ToString("yyyy-MM-dd");
            }
            else
            {
                ultraMaskedEdit1.ResetText();
            }
        }

        private void UpdateMonthCalendar()
        {
            monthCalendar1.DateSelected -= monthCalendar1_DateSelected;
            monthCalendar1.DateChanged -= monthCalendar1_DateChanged;

            if (NullableDate.HasValue)
            {
                monthCalendar1.SetDate(Date);
                monthCalendar1.SetSelectionRange(Date, Date);
            }
            else
            {
                monthCalendar1.SetDate(DefaultNullDate);
                monthCalendar1.SetSelectionRange(DefaultNullDate, DefaultNullDate);
            }

            monthCalendar1.DateSelected += monthCalendar1_DateSelected;
            monthCalendar1.DateChanged += monthCalendar1_DateChanged;
        }

        #endregion

        #region External Events

        [Category("PrimHeure")]
        public event EventHandler ValueChanged;

        private void OnValueChanged()
        {
            if (ReadOnly == false)
            {
                if (ValueChanged != null)
                {
                    ValueChanged(this, EventArgs.Empty);
                }
            }
        }

        [Category("PrimHeure")]
        public event EventHandler KeyPressed_Enter;
        private void OnKeyPressed_Enter()
        {
            if (KeyPressed_Enter != null)
            {
                KeyPressed_Enter(this, EventArgs.Empty);
            }
        }

        #endregion

        #region UI Events

        private void PHDateTimeEditor_Load(object sender, EventArgs e)
        {
        }

        private void ultraMaskedEdit1_Leave(object sender, EventArgs e)
        {
            UpdateValue();
        }

        private void ultraMaskedEdit1_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                UpdateValue();
                OnKeyPressed_Enter();
            }
            else if (e.KeyChar == (char)Keys.Escape)
            {
                UpdateText();
            }
            else if (!(ultraMaskedEdit1.Value is System.DBNull) && !(((string)ultraMaskedEdit1.Value).IsNullOrWhiteSpace()))
            {
                var texts = ultraMaskedEdit1.Text.Split('-');
                if (Char.IsNumber(e.KeyChar))
                {
                    if (ultraMaskedEdit1.SelectionLength != 0)
                    {
                        return;
                    }

                    var cursorIdx = ultraMaskedEdit1.SelectionStart;

                    if (cursorIdx >= 0 && cursorIdx <= 4)
                    {
                        if (texts[0].Length == 4)
                        {
                            if (texts[1].Length == 0)
                            {
                                ultraMaskedEdit1.SelectionStart = 7;
                            }
                            else
                            {
                                e.Handled = true;
                            }
                        }
                    }
                    else if (cursorIdx >= 5 && cursorIdx <= 7)
                    {
                        if (texts[1].Length == 2)
                        {
                            if (texts[2].Length == 0)
                            {
                                ultraMaskedEdit1.SelectionStart = 10;
                            }
                            else
                            {
                                e.Handled = true;
                            }
                        }
                    }
                    else if (cursorIdx >= 8 && cursorIdx <= 10)
                    {
                        if (texts[2].Length == 2)
                        {
                            e.Handled = true;
                        }
                    }
                }
            }
        }

        private void ultraMaskedEdit1_ValueChanged(object sender, EventArgs e)
        {
            if (this.Disposing || this.IsDisposed)
            {
                return;
            }

            if (ultraMaskedEdit1.Value is System.DBNull ||
                ((string)ultraMaskedEdit1.Value).IsNullOrWhiteSpace() || ((string)ultraMaskedEdit1.Value).Length == 10)
            {
                UpdateValue();
            }
        }

        private void ultraMaskedEdit1_Validating(object sender, CancelEventArgs e)
        {
            if (_isValid)
            {
                errorProvider.SetError(this, "");
            }
            else
            {
                errorProvider.SetError(this, "Error!");
                e.Cancel = true;
                this.Focus();
            }
        }

        private void ultraMaskedEdit1_BeforeEditorButtonDropDown(object sender, Infragistics.Win.UltraWinEditors.BeforeEditorButtonDropDownEventArgs e)
        {
            if (ReadOnly)
            {
                e.Cancel = true;
                return;
            }

            monthCalendar1.DateChanged -= monthCalendar1_DateChanged;
            monthCalendar1.DateSelected -= monthCalendar1_DateSelected;

            if (NullableDate.HasValue)
            {
                if (Date >= monthCalendar1.MinDate && Date <= monthCalendar1.MaxDate)
                {
                    monthCalendar1.SetDate(Date);
                }
                else
                {
                    monthCalendar1.SetDate(DateTime.Today);
                }
            }
            else
            {
                monthCalendar1.SetDate(DateTime.Today);
            }
            monthCalendar1.DateChanged += monthCalendar1_DateChanged;
            monthCalendar1.DateSelected += monthCalendar1_DateSelected;
        }

        private void ultraMaskedEdit1_AfterEditorButtonCloseUp(object sender, Infragistics.Win.UltraWinEditors.EditorButtonEventArgs e)
        {
        }

        private void monthCalendar1_DateSelected(object sender, DateRangeEventArgs e)
        {
            ultraMaskedEdit1.Value = e.Start.ToString("yyyy-MM-dd", null);
            ((DropDownEditorButton)ultraMaskedEdit1.ButtonsRight[0]).CloseUp();
        }

        private void monthCalendar1_DateChanged(object sender, DateRangeEventArgs e)
        {
        }

        #endregion
    }
}
